@@ -14,8 +14,8 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import string
 import logging
+import uuid
 import warnings
 
 from six.moves.urllib.parse import parse_qs, urlencode
@@ -37,7 +37,6 @@ def N_(message):
 
 
 SERVICE_URL = 'http://dummy-payment.demo.entrouvert.com/'
-ALPHANUM = string.ascii_letters + string.digits
 LOGGER = logging.getLogger(__name__)
 
 
@@ -88,15 +87,6 @@ class Payment(PaymentCommon):
                 'type': str,
                 'default': 'origin',
             },
-            {
-                'name': 'siret',
-                'caption': 'dummy siret parameter',
-                'validation': lambda x: x.isdigit(),
-                'validation_err_msg': 'must be a number',
-                'type': str,
-                'default': '1234',
-                'required': True,
-            },
             {
                 'name': 'consider_all_response_signed',
                 'caption': (
@@ -149,7 +139,7 @@ class Payment(PaymentCommon):
             '%s amount %s name %s address %s email %s phone %s'
             ' next_url %s info1 %s info2 %s info3 %s kwargs: %s',
             __name__, amount, name, address, email, phone, info1, info2, info3, next_url, kwargs)
-        transaction_id = self.transaction_id(30, ALPHANUM, 'dummy', self.siret)
+        transaction_id = str(uuid.uuid4().hex)
         normal_return_url = self.normal_return_url
         if next_url and not normal_return_url:
             warnings.warn("passing next_url to request() is deprecated, "
@@ -162,7 +152,6 @@ class Payment(PaymentCommon):
             automatic_return_url = self.direct_notification_url
         query = {
             'transaction_id': transaction_id,
-            'siret': self.siret,
             'amount': amount,
             'email': email,
             'return_url': normal_return_url or '',
