@@ -18,14 +18,17 @@ from __future__ import print_function, unicode_literals
 
 import copy
 import datetime
-from decimal import Decimal, ROUND_DOWN
 import functools
 import os
 import random
+import re
+import unicodedata
 import xml.etree.ElementTree as ET
 
 from gettext import gettext as _
 
+import pytz
+
 import six
 from six.moves.urllib.parse import parse_qs
 
@@ -34,7 +37,8 @@ import zeep.exceptions
 
 from .systempayv2 import isonow
 from .common import (PaymentCommon, PaymentResponse, URL, PAID, DENIED,
-                     CANCELLED, ERROR, ResponseError, PaymentException)
+                     CANCELLED, ERROR, ResponseError, PaymentException,
+                     WAITING, EXPIRED, force_text)
 
 WSDL_URL = 'https://www.tipi.budget.gouv.fr/tpa/services/mas_securite/contrat_paiement_securise/PaiementSecuriseService?wsdl'  # noqa: E501
 
@@ -42,6 +46,8 @@ SERVICE_URL = 'https://www.tipi.budget.gouv.fr/tpa/services/securite'  # noqa: E
 
 PAYMENT_URL = 'https://www.tipi.budget.gouv.fr/tpa/paiementws.web'
 
+REFDET_RE = re.compile(r'^[A-Za-z0-9]{1,30}$')
+
 
 def clear_namespace(element):
     def helper(element):
@@ -55,6 +61,19 @@ def clear_namespace(element):
     return element
 
 
+def normalize_objet(objet):
+    '''Make objet a string of 100 chars in alphabet [A-Za-z0-9 ]'''
+    if not objet:
+        return objet
+
+    objet = force_text(objet)
+    objet = unicodedata.normalize('NFKD', objet).encode('ascii', 'ignore').decode()
+    objet = re.sub(r'[\'-]', ' ', objet).strip()
+    objet = re.sub(r'[^A-Za-z0-9 ]', '', objet).strip()
+    objet = re.sub(r'[\s]+', ' ', objet)
+    return objet[:100]
+
+
 class PayFiPError(PaymentException):
     def __init__(self, code, message, origin=None):
         self.code = code
@@ -197,18 +216,40 @@ class Payment(PaymentCommon):
     def _generate_refdet(self):
         return '%s%010d' % (isonow(), random.randint(1, 1000000000))
 
-    def request(self, amount, email, **kwargs):
+    def request(self, amount, email, refdet=None, exer=None, orderid=None,
+                subject=None, transaction_id=None, **kwargs):
         montant = self.clean_amount(amount, max_amount=100000)
 
         numcli = self.numcli
         urlnotif = self.automatic_return_url
         urlredirect = self.normal_return_url
-        exer = str(datetime.date.today().year)
-        refdet = kwargs.get('refdet', self._generate_refdet())
+
+        if not exer:
+            exer = str(datetime.date.today().year)
+
+        if refdet:
+            pass
+        elif transaction_id and REFDET_RE.match(transaction_id):
+            refdet = transaction_id
+        elif orderid and REFDET_RE.match(orderid):
+            refdet = orderid
+        else:
+            refdet = self._generate_refdet()
+
+        objet_parts = []
+        if orderid and refdet != orderid:
+            objet_parts.extend(['O', orderid])
+        if subject:
+            if objet_parts:
+                objet_parts.append('S')
+            objet_parts.append(subject)
+        if transaction_id and refdet != transaction_id:
+            objet_parts.extend(['T', transaction_id])
+        objet = normalize_objet(' '.join(objet_parts))
+
         mel = email
         if hasattr(mel, 'decode'):
             mel = email.decode('ascii')
-
         try:
             if '@' not in mel:
                 raise ValueError('no @ in MEL')
@@ -224,11 +265,52 @@ class Payment(PaymentCommon):
 
         idop = self.payfip.get_idop(numcli=numcli, saisie=saisie, exer=exer,
                                     refdet=refdet, montant=montant, mel=mel,
+                                    objet=objet or None,
                                     url_notification=urlnotif,
                                     url_redirect=urlredirect)
 
         return str(idop), URL, self.payment_url + '?idop=%s' % idop
 
+    def payment_status(self, transaction_id, transaction_date=None, **kwargs):
+        # idop are valid for 15 minutes after their generation
+        # between generation and payment, any call to get_info_paiement() will return a PayFiPError with code=P5
+        # before the end of the 15 minutes it can mean the payment is in progress
+        # after the 15 minutes period it means the payment will never happen,
+        # and after one day the code will change for P1, meaning the idop is
+        # now unknown as it as been cleaned by the night cleaning job.
+        #
+        # So in order to interpret the meaning of PayFiP error codes we need
+        # the date of the start of the transaction and add to it some margin
+        # to.
+        idop = transaction_id
+        if transaction_date:
+            if transaction_date.tzinfo:  # date is aware
+                now = datetime.datetime.now(tz=pytz.utc)
+            else:
+                now = datetime.datetime.now()
+            delta = now - transaction_date
+        else:
+            delta = datetime.timedelta(seconds=0)
+        # set the threshold between transaction 'in progress' and 'expired' at 20 minutes
+        threshold = datetime.timedelta(seconds=20 * 60)
+
+        try:
+            response = self.payfip.get_info_paiement(idop)
+        except PayFiPError as e:
+            if e.code == 'P1' or (
+                    e.code == 'P5' and delta >= threshold):
+                return PaymentResponse(
+                    result=EXPIRED,
+                    signed=True,
+                    order_id=transaction_id)
+            if e.code == 'P5' and delta < threshold:
+                return PaymentResponse(
+                    result=WAITING,
+                    signed=True,
+                    order_id=transaction_id)
+            raise e
+        return self.payfip_response_to_eopayment_response(idop, response)
+
     def response(self, query_string, **kwargs):
         fields = parse_qs(query_string, True)
         idop = (fields.get('idop') or [None])[0]
@@ -236,11 +318,10 @@ class Payment(PaymentCommon):
         if not idop:
             raise ResponseError('missing idop parameter in query string')
 
-        try:
-            response = self.payfip.get_info_paiement(idop)
-        except PayFiPError as e:
-            raise ResponseError('invalid return from payfip', e)
+        return self.payment_status(idop)
 
+    @classmethod
+    def payfip_response_to_eopayment_response(cls, idop, response):
         if response.resultrans == 'P':
             result = PAID
             bank_status = 'paid CB'
