@@ -54,6 +54,7 @@ VADS_TRANS_ID = 'vads_trans_id'
 SIGNATURE = 'signature'
 VADS_CTX_MODE = 'vads_ctx_mode'
 VADS_EFFECTIVE_CREATION_DATE = 'vads_effective_creation_date'
+VADS_EOPAYMENT_TRANS_ID = 'vads_ext_info_eopayment_trans_id'
 
 
 def isonow():
@@ -179,6 +180,7 @@ PARAMETERS = [
 PARAMETER_MAP = dict(((parameter.name,
                        parameter) for parameter in PARAMETERS))
 
+
 def add_vads(kwargs):
     new_vargs = {}
     for k, v in kwargs.items():
@@ -222,6 +224,7 @@ class Payment(PaymentCommon):
             'vads_return_mode': 'NONE'})
 
     '''
+    has_free_transaction_id = True
     service_url = "https://paiement.systempay.fr/vads-payment/"
     signature_algo = 'sha1'
 
@@ -312,7 +315,7 @@ class Payment(PaymentCommon):
     def request(self, amount, name=None, first_name=None, last_name=None,
                 address=None, email=None, phone=None, orderid=None, info1=None,
                 info2=None, info3=None, next_url=None, manual_validation=None,
-                **kwargs):
+                transaction_id=None, **kwargs):
         '''
            Create the URL string to send a request to SystemPay
         '''
@@ -385,9 +388,12 @@ class Payment(PaymentCommon):
         if manual_validation:
             fields['vads_validation_mode'] = '1'
         check_vads(fields)
+        if transaction_id:
+            fields[VADS_EOPAYMENT_TRANS_ID] = transaction_id
+        else:
+            transaction_id = '%s_%s' % (fields[VADS_TRANS_DATE], vads_trans_id)
         fields[SIGNATURE] = force_text(self.signature(fields))
         self.logger.debug('%s request contains fields: %s', __name__, fields)
-        transaction_id = '%s_%s' % (fields[VADS_TRANS_DATE], vads_trans_id)
         self.logger.debug('%s transaction id: %s', __name__, transaction_id)
         form = Form(
             url=self.service_url,
@@ -487,7 +493,10 @@ class Payment(PaymentCommon):
             message += ' signature invalide.'
 
         test = fields[VADS_CTX_MODE] == 'TEST'
-        transaction_id = '%s_%s' % (copy[VADS_TRANS_DATE], copy[VADS_TRANS_ID])
+        if VADS_EOPAYMENT_TRANS_ID in fields:
+            transaction_id = fields[VADS_EOPAYMENT_TRANS_ID]
+        else:
+            transaction_id = '%s_%s' % (copy[VADS_TRANS_DATE], copy[VADS_TRANS_ID])
         # the VADS_AUTH_NUMBER is the number to match payment in bank logs
         copy[self.BANK_ID] = copy.get(VADS_AUTH_NUMBER, '')
         transaction_date = None
@@ -498,7 +507,7 @@ class Payment(PaymentCommon):
             signed=signature_result,
             bank_data=copy,
             order_id=transaction_id,
-            transaction_id=copy.get(VADS_AUTH_NUMBER),
+            transaction_id=transaction_id,
             bank_status=message,
             transaction_date=transaction_date,
             test=test)
