@@ -20,10 +20,11 @@ import mock
 import pytest
 
 import eopayment
+from eopayment.common import PaymentCommon
 
 
 def do_mock_backend(monkeypatch):
-    class MockBackend(object):
+    class MockBackend(PaymentCommon):
         request = mock.Mock()
 
         description = {
@@ -107,3 +108,10 @@ def test_get_parameters(monkeypatch):
     transaction_parameters = payment.get_parameters(scope='transaction')
     assert len(transaction_parameters) == 1
     assert transaction_parameters[0]['name'] == 'manual_validation'
+
+
+def test_payment_status(monkeypatch):
+    _, payment = do_mock_backend(monkeypatch)
+
+    with pytest.raises(NotImplementedError):
+        payment.payment_status('whatever')
